Option Explicit
' $URL: https://opensvn.csie.org/TinyCollection/PSPad/trunk/code/database.vbs $
' $Author: DonQuichote $
' $Date: 2006-08-31 21:50:27 +0200 (Thu, 31 Aug 2006) $
' $Revision: 13 $

' PSPad database routines.
const module_name = "Database"
const module_desc = "Database routines"
const module_ver = "0.4" 'Added multi-command SQL support and separated language-dependent strings.
' 0.4: Added INSERT and UPDATE statements as output formats from database queries.
' 0.4: Parse table names for INSERT writer and UPDATE writer
' ToDo: Add clsScriptRecordWriter and directive recognition: (Dikkie Dik, Sun, 5 March 2006 20:28:17 GMT)
' pspad:database:outputtype = insert/update/lastfield
' pspad:database:command = <command>
' pspad:database:tablename = <name>
' pspad:database:keyfields = <comma separated field list>
' pspad:database:comment = <comment>
' pspad:database:identifierquotes = <two chr string>

'-----------------------------
' SETTINGS. ADAPT TO YOUR NEED.

' If you have enabled the SQL log of your (localhost) database server, give the full path here:
Const SERVER_LOG = ""
'Example: SERVER_LOG = "C:\Program Files\MySQL\MySQL Server 4.1\data\YourSqlLog.sql"

' Your standard ODBC connection. Leave empty if you don't have any
Const ODBC_CONNECTION = ""
'Example for MySQL (You need MyODBC for this): ODBC_CONNECTION = "ODBC;DRIVER={MySQL ODBC 3.51 Driver};SERVER=localhost;PORT=3306;DATABASE=YourDefaultDatabase;UID=YourUserName;PWD=YourPassword;OPTION=18475;"
'Example for SQL Server: ODBC_CONNECTION = "ODBC;DRIVER={SQL Server};UID=YourUserName;PWD=YourPassword;DATABASE=YourDefaultDatabase;SERVER=localhost;"
'for more examples, take a look at http://www.connectionstrings.com/

Const IDENTIFIER_QUOTES = "``"
'For MySQL, set this to IDENTIFIER_QUOTES = "``" (or you might choose IDENTIFIER_QUOTES = """""" if you the server is running in ANSI mode)
'For SQL Server, set this to IDENTIFIER_QUOTES = "[]" or IDENTIFIER_QUOTES = """""".
'ANSI style is IDENTIFIER_QUOTES = """""".

'If True, the "Edit this script" option is added to the menu. Handy when debugging.
Const ADD_EDIT_ME_TO_MENU = True

'If True, a table name is asked even if only one is found from parsing.
Const ALWAYS_ASK_FOR_TABLE_NAMES = False

'If set and the file exists in the PSPad template directory, the contents are output before the query results for the SQL command output.
Const USE_SQL_TEMPLATE = ""
'Example: USE_SQL_TEMPLATE = "Default.sql"

'Language strings
Const cCAPTION_DATABASE_SELECTOR = "Database"
Const cCAPTION_FIELDNAME_SELECTOR = "Key field Names for query"
Const cCAPTION_NO_TABLE_NAMES_FOUND = "No table names were found."
Const cCAPTION_SQL_COMMAND = "Command"
Const cCAPTION_TABLENAME_SELECTOR = "Table name from query"
Const cERR_CANNOT_CONNECT = "Cannot connect to ODBC with "
Const cFILESYSTEM_NOT_AVAILABLE = "Error: Scripting.FileSystemObject could not be instantiated."
Const cERR_NO_CONNECTION = "Please connect first."
Const cERR_NUMBER = "Number"
Const cERR_SOURCE = "Source"
Const cERR_TEMPLATE_NOT_FOUND = "Error: template file %File% could not be found."
Const cMENU_ADD_SLASHES= "Add slashes"
Const cMENU_BEAUTIFY_SQL= "Beautify SQL"
Const cMENU_EDIT_ME = "Edit this script"
Const cMENU_EXTRACT_NAMES = "E&xtract table names"
Const cMENU_OPEN_LOG = "Open SQL log"
Const cMENU_SPLIT_SQL = "Show Split SQL"
Const cMENU_SQL_TO_CSV = "Execute SQL to &Csv"
Const cMENU_SQL_TO_HTML = "Execute SQL to &HTML"
Const cMENU_SQL_TO_INSERTS = "Execute SQL to &INSERT commands"
Const cMENU_SQL_TO_LAST_FIELD = "Execute SQL, but show only &last field"
Const cMENU_SQL_TO_TABS = "Execute SQL to &tabs"
Const cMENU_SQL_TO_UPDATES = "Execute SQL to &UPDATE commands"
Const cTITLE_DATABASE_SELECTOR = "Please state a database"
Const cTITLE_FIELDNAME_SELECTOR = "Name fields in WHERE clause"
Const cTITLE_TABLENAME_SELECTOR = "Name of the table in the UPDATE command"
'    (END OF SETTINGS.)
'-----------------------------

' The manual
' ----------
' This script is designed for accessing a database through ODBC. It is tested with
' MySQL databases, local and remote, with and without a system DSN. It tries to use
' DAO (Data Access Objects, one of Microsoft's many database libraries) or, failing
' that, ADO (Yet another one). If one of these is installed on your system, the
' script should work. If it does not, you may have to install Microsoft's MDAC.
' If ODBC_CONNECTION is left empty, the database access commands will not be shown
' in the menu.
' When asked for a database, you can either give a database name or a connect string
' like in the example above. When you want to use a system DSN (stored ODBC
' connection on your OS), Use a string like "DSN=SomeDsnName;". Don't forget the
' trailing semicolon.
' When you develop on a localhost database server, it is handy to set it to logging
' all SQL. This way, you can always find what you or your database management
' program has done, so you can repeat it on the live database server. If you put
' the full path of this log in SERVER_LOG, it will be accessible through the
' database menu.
' As this script uses classes, VBScript version 5 is needed to run it. It comes
' with Internet Explorer 5.

Const vbTextCompare = 1
const ERR_INVALID_CURSOR = 3670
const ERR_OPERATION_NOT_ALLOWED = 3704
Const SCRIPT_DIRECTIVE = "-- pspad:database:"
Const dbOpenForwardOnly = 8
Const dbUseODBC = 1
Const adOpenForwardOnly = 0
Const adUseClient = 3
Dim LINE_END 'Not a real constant, because functions are called
LINE_END = Chr(13) & Chr(10)

Private mstrCurrentDatabase
Private mobjConnection
Private marrSqlCommands

Class clsAdoConnection
 Private mERRBASE, mERR_NOT_CONNECTED, mcon, mstrSql

 Private Sub Class_Initialize()
     mERRBASE = -2147220750'AC
     mERR_NOT_CONNECTED = mERRBASE + 3
     mstrSql = ""
 End Sub

 Public Sub ExecuteSql(ByVal strSql, ByVal objRecordWriter)
     Dim rst
     If mcon Is Nothing Then
        Err.Raise mERR_NOT_CONNECTED, module_name & ".clsAdoConnection.ExecuteSql", cERR_NO_CONNECTION
     End If
     mstrSql = strSql
     Set rst = CreateObject("ADODB.Recordset")
     Set rst.ActiveConnection = mcon
     On Error Resume Next
     rst.Open strSql, , adOpenForwardOnly
     If Err.Number<>0 Then ShowErrors objRecordWriter, Err.Description
     If ReturnsOutput(rst) Then
        objRecordWriter.SetOriginalSql strSql
        OutputRecordset rst, objRecordWriter
     End If
     On Error Goto 0
 End Sub

 Private Sub OutputRecordset(ByVal rst, ByVal objRecordWriter)
     Dim fld
     objRecordWriter.WriteTableStart
     With rst
          On Error Resume Next
          While Not .EOF
                objRecordWriter.WriteRecordStart
                For Each fld In .Fields
                    objRecordWriter.WriteField fld.Name, fld.Value
                Next
                objRecordWriter.WriteRecordEnd
                .MoveNext
                If Err.Number<>0 Then ShowErrors objRecordWriter, Err.Description
          Wend
          .Close
          On Error Goto 0
     End With
     objRecordWriter.WriteTableEnd
 End Sub

 Private Function RepairConnectString(ByVal strConnect)
     Dim numPos
     numPos = InStr(1, strConnect, "ODBC;")'NOT allowed in ADO connection!
     If numPos>0 Then
        strConnect = Left(strConnect, numPos - 1) & Mid(strConnect, numPos + 5)
     End If
     RepairConnectString = strConnect
 End Function

 Private Function ReturnsOutput(ByVal rst)
     Dim numError, blnDummy
     numError = 0
     On Error Resume Next
     blnDummy = rst.EOF
     numError = Err.Number
     On Error Goto 0
     ReturnsOutput = (numError <> ERR_OPERATION_NOT_ALLOWED)
 End Function

 Private Sub ShowErrors(ByVal objRecordWriter, ByVal strDescription)
     Dim objError
     objRecordWriter.WriteMessage strDescription & "/" & Err.Source
     MsgBox strDescription
     For Each objError In mcon.Errors
         objRecordWriter.WriteMessage objError.Description & " (" &  cERR_NUMBER & " " & objError.Number & ", " & cERR_SOURCE & ": " & objError.Source & ")"
     Next
     If mstrSql<>"" Then
        objRecordWriter.WriteMessage cCAPTION_SQL_COMMAND & ": " & mstrSql
     End If
 End Sub

 Public Function TryToConnect(ByVal strConnect)
     Set mcon = CreateObject("ADODB.Connection")
     mstrSql = ""
     If Not (mcon Is Nothing) Then
        On Error Resume Next
        mcon.Open RepairConnectString(strConnect)
        If Err.Number<>0 Then
           Set mcon = Nothing
        End If
        On Error Goto 0
     End If
     TryToConnect = Not (mcon Is Nothing)
 End Function
End Class

Class clsDaoConnection
 Private mERRBASE, mERR_NOT_CONNECTED, mobjRoot, mwsp, mcon, mstrSql

 Private Sub Class_Initialize()
     mERRBASE = -2147218722'DC
     mERR_NOT_CONNECTED = mERRBASE + 3
     mstrSql = ""
 End Sub

 Public Sub ExecuteSql(ByVal strSql, ByVal objRecordWriter)
     Dim rst, qry
     If mcon Is Nothing Then
        Err.Raise mERR_NOT_CONNECTED, module_name & ".clsDaoConnection.ExecuteSql", cERR_NO_CONNECTION
     End If
     On Error Resume Next
     Set qry = mcon.CreateQueryDef("PSPadScript", strSql)
     If Err.Number<>0 Then ShowErrors objRecordWriter, Err.Description
     Set rst = qry.OpenRecordset(dbOpenForwardOnly)
     If Err.Number<>0 Then ShowErrors objRecordWriter, Err.Description
     If ReturnsOutput(rst) Then
        objRecordWriter.SetOriginalSql strSql
        OutputRecordset rst, objRecordWriter, strSql
     End If
     On Error Goto 0
 End Sub

 Private Sub OutputRecordset(ByVal rst, ByVal objRecordWriter)
     Dim fld
     objRecordWriter.WriteTableStart
     With rst
          On Error Resume Next
          While Not .EOF
                objRecordWriter.WriteRecordStart
                For Each fld In .Fields
                    objRecordWriter.WriteField fld.Name, fld.Value
                Next
                objRecordWriter.WriteRecordEnd
                .MoveNext
                If Err.Number<>0 Then ShowErrors objRecordWriter, Err.Description
          Wend
          .Close
          On Error Goto 0
     End With
     objRecordWriter.WriteTableEnd
 End Sub

 Private Function RepairConnectString(ByVal strConnect)
     Dim numPos
     numPos = InStr(1, strConnect, "ODBC;")'MUST be present in DAO ODBC connection
     If numPos<1 Then
        strConnect = "ODBC;" & strConnect
     End If
     RepairConnectString = strConnect
 End Function

 Private Function ReturnsOutput(ByVal rst)
     Dim numError, blnDummy
     numError = 0
     On Error Resume Next
     blnDummy = rst.EOF
     numError = Err.Number
     On Error Goto 0
     ReturnsOutput = (numError <> ERR_INVALID_CURSOR)
 End Function

 Private Sub ShowErrors(ByVal objRecordWriter, ByVal strDescription)
     Dim objError
     objRecordWriter.WriteMessage Err.Description
     For Each objError In mobjRoot.Errors
         objRecordWriter.WriteMessage objError.Description & " (" & Number & " " & objError.Number & ", " & cERR_SOURCE & ": " & objError.Source & ")"
     Next
     If mstrSql<>"" Then
        objRecordWriter.WriteMessage cCAPTION_SQL_COMMAND & ": " & mstrSql
     End If
 End Sub

 Public Function TryToConnect(ByVal strConnect)
     Set mcon = Nothing
     mstrSql = ""
     Set mobjRoot = CreateObject("DAO.DBEngine.35") ' DAO 3.51
     If mobjRoot Is Nothing Then
        Set mobjRoot = CreateObject("DAO.DBEngine.36") ' DAO 3.6
     End If
     If Not (mobjRoot Is Nothing) Then
        Set mwsp = mobjRoot.CreateWorkspace("PSPad", "", "", dbUseODBC)
        If Not (mwsp Is Nothing) Then
           On Error Resume Next
           Set mcon = mwsp.OpenConnection("", , , RepairConnectString(strConnect))
           On Error Goto 0
        End If
     End If
     TryToConnect = Not (mcon Is Nothing)
 End Function
End Class

Class clsLiteralBuilder
 Private Function PadTwo(ByVal strNumber)
     While Len(strNumber)<2
           strNumber = "0" & strNumber
     Wend
     PadTwo = strNumber
 End Function

 Public Function ToLiteral(ByVal varValue)
     Select Case TypeName(varValue)
            Case "Byte", "Integer", "Long", "Error"
                 ToLiteral = FormatNumber(varValue, 0)
            Case "Single", "Double", "Currency", "Decimal"
                 ToLiteral = TrimTrailingZeros(FormatNumber(varValue, 15, -1, 0, 0))
            Case "String"
                 ToLiteral = AddSlashesTo(varValue)
            Case "Boolean"
                 If varValue Then
                    ToLiteral = "1"
                 Else
                    ToLiteral = "0"
                 End If
            Case "Date"
                 ToLiteral = "'" & DatePart("yyyy", varValue) & "-" & PadTwo(DatePart("m", varValue)) & "-" & PadTwo(DatePart("d", varValue)) & " " & PadTwo(DatePart("h", varValue)) & ":" & PadTwo(DatePart("n", varValue)) & ":" & PadTwo(DatePart("s", varValue)) & "'"
            Case "Null", "Empty", "Nothing"
                 ToLiteral = "NULL"
            Case Else
                 ToLiteral = "*unknown*"
     End Select
 End Function

 Private Function TrimTrailingZeros(ByVal strNumber)
     While Right(strNumber, 1)="0"
           strNumber = Left(strNumber, Len(strNumber) - 1)
     Wend
     If Right(strNumber, 1)="." Then
        strNumber = Left(strNumber, Len(strNumber) - 1)
     End If
     TrimTrailingZeros = strNumber
  End Function
End Class

Class clsCsvRecordWriter
 Private mblnIsGatheringHeaders, mstrHeaders, mstrValues, medtLog, mstrQuote

 Private Sub Class_Initialize()
     Set medtLog = newEditor
     medtLog.assignLog
     medtLog.text ""
     mstrQuote = """"
 End Sub

 Public Sub SetComment(ByVal strCommentLine)
     'Not used. Only exists for interface compatibility
 End Sub

 Public Sub SetOriginalSql(ByVal strSql)
     'Not used. Only exists for interface compatibility
 End Sub

 Public Property Get UsesTableName()
     UsesTableName = False
 End Property

 Public Sub WriteCommand(ByVal strCommand)
     'No commands exists for CVSoutput
 End Sub

 Public Sub WriteField(ByVal strName, ByVal varValue)
     mstrValues = mstrValues & "," & mstrQuote & varValue & mstrQuote
     If mblnIsGatheringHeaders Then
        mstrHeaders = mstrHeaders & "," & mstrQuote & strName & mstrQuote
     End If
 End Sub

 Public Sub WriteMessage(ByVal strMessage)
     medtLog.appendText strMessage & LINE_END
 End Sub

 Public Sub WriteRecordStart()
     mstrValues = ""
 End Sub

 Public Sub WriteRecordEnd()
     If mblnIsGatheringHeaders Then
        medtLog.appendText Mid(mstrHeaders, 2) & LINE_END
        mblnIsGatheringHeaders = False
     End If
     medtLog.appendText Mid(mstrValues, 2) & LINE_END
 End Sub

 Public Sub WriteTableStart()
     mblnIsGatheringHeaders = True
     mstrHeaders = ""
 End Sub

 Public Sub WriteTableEnd()
 End Sub
End Class

Class clsLastFieldRecordWriter
 Private mstrValues, medtLog

 Private Sub Class_Initialize()
     Set medtLog = newEditor
     medtLog.assignLog
     medtLog.text ""
     mstrValues = ""
 End Sub

 Public Sub SetComment(strCommentLine)
     'Not used. Only exists for interface compatibility
 End Sub

 Public Sub SetOriginalSql(ByVal strSql)
     'Not used. Only exists for interface compatibility
 End Sub

 Public Property Get UsesTableName()
     UsesTableName = False
 End Property

 Public Sub WriteCommand(ByVal strCommand)
     medtLog.appendText strCommand & LINE_END
 End Sub

 Public Sub WriteField(ByVal strName, ByVal varValue)
     mstrValues = varValue
 End Sub

 Public Sub WriteMessage(ByVal strMessage)
     medtLog.appendText LINE_END & "-- " & strMessage & LINE_END
 End Sub

 Public Sub WriteRecordStart()
     mstrValues = ""
 End Sub

 Public Sub WriteRecordEnd()
     medtLog.appendText mstrValues & LINE_END
 End Sub

 Public Sub WriteTableStart()
    With medtLog
         If Len(.text())>0 Then
            .appendText LINE_END ' Separate query outputs by an empty line.
         End If
    End With
 End Sub

 Public Sub WriteTableEnd()
 End Sub
End Class

Class clsTabbedRecordWriter
 Private mblnIsGatheringHeaders, mstrHeaders, mstrValues, medtLog, mstrTab

 Private Sub Class_Initialize()
     Set medtLog = newEditor
     medtLog.assignLog
     medtLog.text ""
     mstrTab = chr(9)
 End Sub

 Public Sub SetComment(strCommentLine)
     'Not used. Only exists for interface compatibility
 End Sub

 Public Sub SetOriginalSql(ByVal strSql)
     'Not used. Only exists for interface compatibility
 End Sub

 Public Sub WriteCommand(ByVal strCommand)
     'No commands exists for tabbed output
 End Sub

 Public Sub WriteField(ByVal strName, ByVal varValue)
     mstrValues = mstrValues & mstrTab & varValue
     If mblnIsGatheringHeaders Then
        mstrHeaders = mstrHeaders & mstrTab & strName
     End If
 End Sub

 Public Sub WriteMessage(ByVal strMessage)
     medtLog.appendText LINE_END & strMessage & LINE_END & LINE_END
 End Sub

 Public Sub WriteRecordStart()
     mstrValues = ""
 End Sub

 Public Sub WriteRecordEnd()
     If mblnIsGatheringHeaders Then
        medtLog.appendText Mid(mstrHeaders, 2) & LINE_END
        mblnIsGatheringHeaders = False
     End If
     medtLog.appendText Mid(mstrValues, 2) & LINE_END
 End Sub

 Public Sub WriteTableStart()
     mblnIsGatheringHeaders = True
     mstrHeaders = ""
 End Sub

 Public Sub WriteTableEnd()
 End Sub
End Class

Class clsTableRecordWriter
 Private mblnIsGatheringHeaders, mstrHeaders, mstrValues, medtLog

 Private Sub Class_Initialize()
     Set medtLog = newEditor
     medtLog.assignLog
     medtLog.text ""
 End Sub

 Public Sub SetComment(strCommentLine)
     'Not used. Only exists for interface compatibility
 End Sub

 Public Sub SetOriginalSql(ByVal strSql)
     'Not used. Only exists for interface compatibility
 End Sub

 Public Sub WriteCommand(ByVal strCommand)
     medtLog.appendText strCommand & LINE_END
 End Sub

 Public Sub WriteField(ByVal strName, ByVal varValue)
     mstrValues = mstrValues & "    <td>" & varValue & "</td>" & LINE_END
     If mblnIsGatheringHeaders Then
        mstrHeaders = mstrHeaders & "    <th>" & strName & "</th>" & LINE_END
     End If
 End Sub

 Public Sub WriteMessage(ByVal strMessage)
     medtLog.appendText LINE_END & "<strong>" & strMessage & "</strong><br>" & LINE_END & LINE_END
 End Sub

 Public Sub WriteRecordStart()
     mstrValues = "  <tr>" & LINE_END
 End Sub

 Public Sub WriteRecordEnd()
     mstrValues = mstrValues & "  </tr>" & LINE_END
     If mblnIsGatheringHeaders Then
        medtLog.appendText mstrHeaders & "  </tr>" & LINE_END
        mblnIsGatheringHeaders = False
     End If
     medtLog.appendText mstrValues
 End Sub

 Public Sub WriteTableStart()
     mblnIsGatheringHeaders = True
     mstrHeaders = "<table>" & LINE_END & "  <tr>" & LINE_END
 End Sub

 Public Sub WriteTableEnd()
     medtLog.appendText "</table>" & LINE_END
 End Sub
End Class

Class clsSqlInsertRecordWriter
 Private mblnIsGatheringHeaders, mstrHeaders, mstrValues, medtLog, mstrTableName, mobjLiteralBuilder, mstrOriginalSql

 Private Sub Class_Initialize()
     Set medtLog = newEditor
     medtLog.assignLog
     With New clsSqlTemplate
          medtLog.text .Contents()
     End With
     mstrTableName = "YourTableHere"
     Set mobjLiteralBuilder = New clsLiteralBuilder
 End Sub

 Private Function ContainsSpace(ByVal strText)
     ContainsSpace = (InStr(1, strText, " ")>0)
 End Function

 Private Function QuoteIfNecessary(ByVal strField)
     If ContainsSpace(strField) Or IsNumeric(strField) Then
        strField = Left(IDENTIFIER_QUOTES, 1) & strField & Mid(IDENTIFIER_QUOTES, 2, 1)
     End If
     QuoteIfNecessary = strField
 End Function

 Public Sub SetComment(strCommentLine)
     'Not used Yet.
 End Sub

 Public Sub SetOriginalSql(ByVal strSql)
     mstrOriginalSql = strSql
 End Sub

 Public Sub WriteCommand(ByVal strCommand)
     medtLog.appendText strCommand & LINE_END
 End Sub

 Public Sub WriteField(ByVal strName, ByVal varValue)
     mstrValues = mstrValues & ", " & mobjLiteralBuilder.ToLiteral(varValue)
     If mblnIsGatheringHeaders Then
        mstrHeaders = mstrHeaders & "," & QuoteIfNecessary(strName)
     End If
 End Sub

 Public Sub WriteMessage(ByVal strMessage)
     medtLog.appendText LINE_END & "-- " & strMessage & LINE_END
 End Sub

 Public Sub WriteRecordStart()
     mstrValues = ""
 End Sub

 Public Sub WriteRecordEnd()
     If mblnIsGatheringHeaders Then
        mstrHeaders ="INSERT INTO " & mstrTableName & "(" & Mid(mstrHeaders, 2) & ") VALUES("
        mblnIsGatheringHeaders = False
     End If
     medtLog.appendText mstrHeaders & Mid(mstrValues, 3) & ");" & LINE_END
 End Sub

 Public Sub WriteTableStart()
     mblnIsGatheringHeaders = True
     mstrHeaders = ""
     With medtLog
          If Len(.text())>0 Then
             .appendText LINE_END ' Separate query outputs by an empty line.
          End If
     End With
     With New clsTableExtractor
          .SetSql mstrOriginalSql, ALWAYS_ASK_FOR_TABLE_NAMES
          mstrTableName = .AskForTable()
     End With
 End Sub

 Public Sub WriteTableEnd()
 End Sub
End Class

Class clsSqlUpdateRecordWriter
 Private mstrValues, medtLog, mstrTableName, mobjLiteralBuilder, mstrKeyField, mstrKeyValues, marrKeyField, mstrOriginalSql

 Public Sub AddKeyField(ByVal strName)
    Dim numCount
    numCount = UBound(marrKeyField)
    marrKeyField(numCount) = strName
    Redim Preserve marrKeyField(numCount + 1)
 End Sub

 Private Sub Class_Initialize()
     Set medtLog = newEditor
     medtLog.assignLog
     With New clsSqlTemplate
          medtLog.text .Contents()
     End With
     mstrTableName = "YourTableHere"
     mstrOriginalSql = ""
     Redim marrKeyField(0)
     Set mobjLiteralBuilder = New clsLiteralBuilder
 End Sub

 Private Function ContainsSpace(ByVal strText)
     ContainsSpace = (InStr(1, strText, " ")>0)
 End Function

 Private Function IsKeyField(ByVal strName)
     Dim numCount, blnResult
     strName = UCase(strName)
     blnResult = False
     numCount = UBound(marrKeyField) - 1
     While numCount>=0
           If strName = UCase(marrKeyField(numCount)) Then
              blnResult = True
              numCount = -1
           End If
           numCount = numCount - 1
     Wend
     IsKeyField = blnResult
 End Function

 Private Function QuoteIfNecessary(ByVal strField)
     If ContainsSpace(strField) Or IsNumeric(strField, 1) Then
        strField = Left(IDENTIFIER_QUOTES, 1) & strField & Mid(IDENTIFIER_QUOTES, 2, 1)
     End If
     QuoteIfNecessary = strField
 End Function

 Public Sub SetComment(strCommentLine)
     'Not used yet.
     ' ToDo: Implement identifier directives (Dikkie Dik, Mon, 13 March 2006 07:05:55 GMT)
 End Sub

 Public Sub SetOriginalSql(ByVal strSql)
     mstrOriginalSql = strSql
 End Sub

 Public Sub WriteCommand(ByVal strCommand)
     medtLog.appendText strCommand & LINE_END
 End Sub

 Public Sub WriteField(ByVal strName, ByVal varValue)
     If IsKeyField(strName) Then
        mstrKeyValues = mstrKeyValues & " AND " & QuoteIfNecessary(strName) & "=" & mobjLiteralBuilder.ToLiteral(varValue)
     Else
        mstrValues = mstrValues & ", " & QuoteIfNecessary(strName) & "=" & mobjLiteralBuilder.ToLiteral(varValue)
     End If
 End Sub

 Public Sub WriteMessage(ByVal strMessage)
     medtLog.appendText LINE_END & "-- " & strMessage & LINE_END
 End Sub

 Public Sub WriteRecordStart()
     mstrValues = ""
     mstrKeyValues = ""
 End Sub

 Public Sub WriteRecordEnd()
     medtLog.appendText "UPDATE " & mstrTableName & " SET " & Mid(mstrValues, 3) & " WHERE " & Mid(mstrKeyValues, 6) & ";" & LINE_END
 End Sub

 Public Sub WriteTableStart()
     With New clsTableExtractor
          .SetSql mstrOriginalSql, ALWAYS_ASK_FOR_TABLE_NAMES
          mstrTableName = .AskForTable()
     End With
     With New clsKeyFieldReader
          .SetSql mstrOriginalSql
          .ReadFieldsTo Me
     End With
     With medtLog
          If Len(.text())>0 Then
             .appendText LINE_END ' Separate query outputs by an empty line.
          End If
     End With
 End Sub

 Public Sub WriteTableEnd()
     Redim marrKeyField(0) ' Clear the key fields for the next query.
 End Sub
End Class

Class clsArrayWriter
 Private medtLog

 Private Sub Class_Initialize()
     Set medtLog = newEditor()
     medtLog.assignLog()
     medtLog.text ""
 End Sub

 Public Sub WriteLine(ByVal strLine)
     medtLog.appendText(strLine & chr(13) & chr(10))
 End Sub
End Class

Class clsSingleLineArrayWriter
 Private mstrOutput

 Private Sub Class_Initialize()
     mstrOutput = ""
 End Sub

 Public Function ResultLine()
     ResultLine = Trim(mstrOutput)
 End Function

 Public Sub WriteLine(ByVal strLine)
     mstrOutput = mstrOutput & " " & strLine
 End Sub
End Class

Class clsDefaultMatchProcessor
 Function Process(strInput)
     Process = strInput
 End Function
End Class

Class clsDeleteMatchProcessor
 Function Process(ByVal strInput)
     Dim numPos
     strInput = Trim(strInput)
     numPos = InStr(1, strInput, "FROM", vbTextCompare)
     If numPos>0 Then
        strInput = Trim(Mid(strInput, numPos + 5))
     End If
     Process = strInput
 End Function
End Class

Class clsFirstWordMatchProcessor
 Function Process(ByVal strInput)
     Dim numPos
     strInput = Trim(strInput)
     numPos = InStr(1, strInput, " ")
     If numPos>0 Then
        strInput = Rtrim(Left(strInput, numPos - 1))
     End If
     Process = strInput
 End Function
End Class

Class clsHandlerMatchProcessor
 Function Process(ByVal strInput)
     Process = Trim(Mid(strInput,8,Len(strInput)-11))
 End Function
End Class

Class clsJoinMatchProcessor
 Function Process(ByVal strInput)
     Process = Trim(Mid(Trim(strInput),6))
 End Function
End Class

Class clsLastWordMatchProcessor
 Function Process(ByVal strInput)
     Dim numPos
     strInput = Trim(strInput)
     numPos = InStrRev(strInput, " ")
     If numPos>0 Then
        strInput = Mid(strInput, numPos + 1)
     End If
     Process = strInput
 End Function
End Class

Class clsUpdateMatchProcessor
 Function Process(ByVal strInput)
     Dim numPos
     strInput = Trim(strInput)
     numPos = InStr(1, strInput, "UPDATE", vbTextCompare)
     If numPos>0 Then
        strInput = Ltrim(Mid(strInput, numPos + 6))
     End If
     numPos = InStrRev(strInput, "SET", -1, vbTextCompare)
     If numPos>0 Then
        strInput = Rtrim(Left(strInput, numPos - 1))
     End If
     Process = strInput
 End Function
End Class

Class clsResultGatherer
 private marrResult()
 Private medtCurrent
 private mstrTextToInvestigate

 Private Sub Add(ByVal strResult)
     Dim numCount
     If strResult<>"" Then
        If Not AlreadyPresent(strResult) Then
           numCount = UBound(marrResult)
           marrResult(numCount) = strResult
           Redim Preserve marrResult(numCount + 1)
        End If
     End If
 End Sub

 Private Property Get AlreadyPresent(ByVal strText)
     Dim blnFound, numCount
     blnFound = False
     numCount = UBound(marrResult)-1
     While numCount>=0
           If marrResult(numCount)=strText Then
              blnFound = True
              numCount = -5
           End If
           numCount = numCount-1
     Wend
     AlreadyPresent = blnFound
 End Property

 Private Sub Class_Initialize()
     Redim marrResult(0)
     Set medtCurrent = newEditor()
     medtCurrent.assignActiveEditor()
     mstrTextToInvestigate = medtCurrent.text()
 End Sub

 Public Sub SetTextToInvestigate(ByVal strText)
     mstrTextToInvestigate = strText
 End Sub

 Public Sub Process(ByVal strRegExp, ByVal objProcessor)
     Dim objMatch
     With New RegExp
          .Pattern = strRegExp
          .IgnoreCase = True
          .Global = True
          For Each objMatch In .Execute(mstrTextToInvestigate)
              Add objProcessor.Process(objMatch.Value)
          Next
     End With
 End Sub

 Public Sub WriteTo(ByVal objWriter)
     Dim numCount
     For numCount = 0 To UBound(marrResult)-1
         objWriter.WriteLine(marrResult(numCount))
     Next
     If UBound(marrResult)=0 Then
        objWriter.WriteLine(cCAPTION_NO_TABLE_NAMES_FOUND)
     End If
 End Sub
End Class

Class clsSqlSplitter
 Private marrLines()

 Private Sub Class_Initialize()
     Redim marrLines(0)
 End Sub

 Private Sub Add(ByVal strSql)
     Dim numCount
     numCount = UBound(marrLines)
     If Left(strSql, 3)="-- " Then
        If LCase(Left(strSql, 18))<>SCRIPT_DIRECTIVE Then
           strSql = "" ' Not a directive, but a comment: effectively an empty line
        End If
     End If
     If strSql<>"" Then
        marrLines(numCount) = strSql
        Redim Preserve marrLines(numCount + 1)
     End If
 End Sub

 Public Sub AddSql(ByVal strSql)
     Dim numPosition, strCurrentQuote, strChr
     strCurrentQuote = ""
     strSql = WithoutLeadingLineEnd(strSql)
     numPosition = 1
     While numPosition<=Len(strSql)
           strChr = Mid(strSql, numPosition, 1)
           Select Case strChr
                 Case "'", """":
                      If strCurrentQuote="" Then
                         strCurrentQuote = strChr
                      Else
                        If strCurrentQuote=strChr Then
                           strCurrentQuote = ""
                        End If
                      End If
                 Case ";":
                      If strCurrentQuote="" Then
                         Add Left(strSql, numPosition - 1)
                         strSql = WithoutLeadingLineEnd(Mid(strSql, numPosition + 1))
                         numPosition = 0
                      End If
                 Case Chr(13), Chr(10):
                      If numPosition=1 Then
                         strSql = WithoutLeadingLineEnd(Mid(strSql, 2))
                      Else
                        If Left(strSql, 3)="-- " Then
                           strSql = WithoutLeadingLineEnd(Mid(strSql, numPosition + 1))
                           numPosition = 0
                        End If
                      End If
           End Select
           numPosition = numPosition + 1
     Wend
     Add strSql
 End Sub

 Public Sub Execute(ByVal objConnection, ByVal objRecordWriter)
     Dim numCount
     For numCount = 0 To UBound(marrLines) - 1
         If Left(marrLines(numCount), 3)="-- " Then
            objRecordWriter.SetComment marrLines(numCount)
         Else
            objConnection.ExecuteSql marrLines(numCount), objRecordWriter
         End If
     Next
 End Sub

 Private Function WithoutLeadingLineEnd(ByVal strLine)
     While (Left(strLine,1)=Chr(13)) OR (Left(strLine,1)=Chr(10))
           strLine = Trim(Mid(strLine, 2))
     Wend
     WithoutLeadingLineEnd = Trim(strLine)
 End Function

 Public Sub WriteResults(ByVal objRecordWriter)
     Dim numCount
     With objRecordWriter
          .WriteTableStart
          For numCount = 0 To UBound(marrLines)
              .WriteRecordStart
              .WriteField cCAPTION_SQL_COMMAND, marrLines(numCount) & ";"
              .WriteRecordEnd
          Next
          .WriteTableEnd
     End With
 End Sub
End Class

Class clsAccesskeyGenerator
 Dim mstrKeys

 Private Sub AddKey(ByVal strKey)
     mstrKeys = mstrKeys & UCase(strKey)
 End Sub

 Private Sub Class_Initialize()
     mstrKeys = ""
 End Sub

 Private Sub HandleGivenAccessKey(ByRef strCommand, ByRef numPos)
     If Not IsKeyKnown(Mid(strCommand, numPos + 1, 1)) Then
        AddKey Mid(strCommand, numPos + 1, 1)
     Else
        strCommand = Left(strCommand, numPos - 1) & Mid(strCommand, numPos + 1)
        numPos = 0 ' remove accesskey, as it was already registered
     End If
 End Sub

 Private Sub DetermineNewAccessKey(ByRef strCommand)
     Dim numPos
     numPos = 1
     While IsKeyKnown(Mid(strCommand, numPos, 1)) And (numPos<=Len(strCommand))
           numPos = numPos + 1
     Wend
     If numPos<=Len(strCommand) Then
        AddKey Mid(strCommand, numPos, 1)
        strCommand = Left(strCommand, numPos - 1) & "&" & Mid(strCommand, numPos)
     End If
 End Sub

 Private Function IsKeyKnown(ByVal strKey)
     IsKeyKnown = InStr(1, mstrKeys, UCase(strKey))>0
 End Function

 Public Function WithAccessKey(ByVal strCommand)
     Dim numPos, strKey
     numPos = InStr(1, strCommand, "&")
     If numPos>0 Then
        HandleGivenAccessKey strCommand, numPos
     End If
     If numPos<1 Then
        DetermineNewAccessKey strCommand
     End If
     WithAccessKey = strCommand
 End Function
End Class

Class clsDirectiveInterpreter
 private mstrComment, mstrIdentifierQuotes, mstrTableName, mstrOutputType, mstrCommand, mstrKeyFields

 Private Sub AddCommentLine(ByVal strCommentLine)
  If strCommentLine<>"" Then
     Dim numCount
     numCount = UBound(mstrComment)
     mstrComment(numCount) = strCommentLine
     Redim Preserve mstrComment(numCount + 1)
  End If
 End Sub

 Private Sub Class_Initialize()
     ReInitialize
     mstrIdentifierQuotes = IDENTIFIER_QUOTES ' "Sticky" options: remembered between SQL commands.
     mstrOutputType = "insert"
 End Sub

 Public Sub Dispose()
     ' ToDo: Forget the current ouputwriter (Dikkie Dik, Sun, 12 March 2006 20:34:24 GMT)
 End Sub

 Private Function IsOption(ByVal strText, ByVal strOptionName)
     Dim blnResult, strParsedOption, numPos
     blnResult = True
     numPos = InStr(1, strText, "=")
     If numPos>1 Then
        strParsedOption = LCase(Trim(Left(strText, numPos - 1)))
        blnResult = (LCase(strOptionName)=strParsedOption)
     Else
        blnResult = False
     End If
     IsOption = blnResult
 End Function

 Private Function OptionValue(ByVal strText)
     Dim numPos
     numPos = InStr(1, strText, "=")
     If numPos>1 Then
        strText = Trim(Mid(strText, numPos + 1))
     Else
        strText = ""
     End If
     OptionValue = strText
 End Function

 Private Sub ReadCommentsTo(ByVal objResultWriter)
     Dim numIndex
     For numIndex = LBound(mstrComment) To UBound(mstrComment)-1
         objResultWriter.WriteMessage mstrComment(numIndex)
     Next
 End Sub

 Private Sub ReInitialize()
     Redim mstrComment(0)
     mstrTableName = ""
     mstrCommand = ""
     mstrKeyFields = ""
 End Sub

 Public Sub SetComment(ByVal strCommentLine)
     If LCase(Left(strCommentLine, Len(SCRIPT_DIRECTIVE))) = LCase(SCRIPT_DIRECTIVE) Then
        strCommentLine = Trim(Mid(strCommentLine, Len(SCRIPT_DIRECTIVE) + 1))
     End If
     If IsOption(strCommentLine, "tablename") Then
        mstrTableName = OptionValue(strCommentLine)
     ElseIf IsOption(strCommentLine, "comment") Then
        AddCommentLine OptionValue(strCommentLine)
     ElseIf IsOption(strCommentLine, "outputtype") Then
        SetOutputType OptionValue(strCommentLine)
     ElseIf IsOption(strCommentLine, "identifierquotes") Then
        SetIdentifierQuotes OptionValue(strCommentLine)
     End If
 End Sub

 Private Sub SetIdentifierQuotes(ByVal strQuotes)
     mstrIdentifierQuotes = strQuotes
     If Len(mstrIdentifierQuotes)=1 Then
        mstrIdentifierQuotes = mstrIdentifierQuotes & mstrIdentifierQuotes
     End If
 End Sub
' ToDo: Error message in else clause (Dikkie Dik, Sat, 11 March 2006 18:49:31 GMT)
 Private Sub SetOutputType(ByVal strType)
     strType = LCase(strType)
     If (strType="insert") Or (strType="update") Or (strType="lastfield") Then
        mstrOutputType = strType
     End If
 End Sub
End Class

Class clsTableExtractor
 Private mstrSql, mysnSilentWhenPossible

 Public Function AskForTable()
     Dim objTableGetter, strResult
     Set objTableGetter = New clsSingleLineArrayWriter
     With New clsResultGatherer
          .SetTextToInvestigate mstrSql
          .Process "\s(FROM|JOIN)\s+(\w+)", New clsJoinMatchProcessor
          .WriteTo objTableGetter
     End With
     strResult = objTableGetter.ResultLine
     If ContainsSpace(strResult) Or ALWAYS_ASK_FOR_TABLE_NAMES Then
        AskForTable = InputBox(cCAPTION_TABLENAME_SELECTOR & LINE_END & mstrSql, cTITLE_TABLENAME_SELECTOR, strResult)
     Else
        AskForTable = strResult
     End If
 End Function

 Private Function ContainsSpace(ByVal strText)
     ContainsSpace = (InStr(1, strText, " ")>0)
 End Function

 Private Sub Class_Initialize()
     mstrSql = ""
     mysnSilentWhenPossible = True
 End Sub

 Public Sub SetSql(ByVal strSql, ByVal ysnSilentWhenPossible)
     mstrSql = strSql
     mysnSilentWhenPossible = ysnSilentWhenPossible
 End Sub
End Class

Class clsKeyFieldReader
 Private mstrSql

 Private Sub Class_Initialize()
     Redim mstrKeyFields(0)
     mstrSql = ""
 End Sub

 Private Function ParameterAfterCleanup(ByVal strParameter)
     Dim numPos
     strParameter = Trim(strParameter)
     If Len(strParameter)>1 Then
        numPos = InStr(1, UCase(strParameter), " AS ")
        If numPos>0 Then
           strParameter = Trim(Mid(strParameter, numPos + 4))
        End If
        If UCase(Left(strParameter, 4))="TOP " Then
           strParameter = Trim(Mid(strParameter, 5))
           While IsNumeric(Left(strParameter, 1))
                 strParameter = Mid(strParameter, 2)
           Wend
           strParameter = Trim(strParameter)
        End If
     End If
     ParameterAfterCleanup = strParameter
 End Function

 Private Function CleanUpParameterList(ByVal strList)
     Dim arrFields, numIndex
     arrFields = Split(strList, ",")
     For numIndex = LBound(arrFields) To UBound(arrFields)
         arrFields(numIndex) = ParameterAfterCleanup(arrFields(numIndex))
     Next
     CleanUpParameterList = Join(arrFields, ", ")
 End Function

 Private Function DefaultList()
     Dim strResult, numPos, arrFields, numIndex
     strResult = ""
     If UCase(Left(Trim(mstrSql), 6))="SELECT" Then
        strResult = Trim(Mid(Trim(mstrSql), 7))
        strResult = Replace(strResult, Chr(13), " ")
        strResult = Replace(strResult, Chr(10), " ")
        numPos = InStr(1, strResult, " FROM ")
        If numPos>0 Then
           strResult = Left(strResult, numPos - 1)
        End If
        strResult = CleanUpParameterList(strResult)
     End If
     DefaultList = strResult
 End Function

 Public Sub ReadFieldsTo(ByVal objUpdateRecordWriter)
     Dim numCount, strFields, arrKeyFields
     If Len(Trim(mstrSql))>0 Then
        strFields = DefaultList()
        strFields = InputBox(cCAPTION_FIELDNAME_SELECTOR & LINE_END & mstrSql, cTITLE_FIELDNAME_SELECTOR, strFields)
        arrKeyFields = Split(strFields, ",")
        For numCount = 0 To UBound(arrKeyFields)
            objUpdateRecordWriter.AddKeyField Trim(arrKeyFields(numCount))
        Next
     End If
 End Sub

 Public Sub SetSql(ByVal strSql)
     mstrSql = strSql
 End Sub
End Class

Class clsSqlTemplate
 Private mstrContents
 Private mblnWasRead

 Private Sub Class_Initialize()
     mblnWasRead = False
     mstrContents = ""
 End Sub

 Public Function Contents()
     If Not mblnWasRead Then
        SetContents
     End If
     Contents = mstrContents
 End Function

 Private Sub SetContents()
    Const FOR_READING = 1
    If Len(USE_SQL_TEMPLATE)>0 Then
       Dim strFile, objFileSystem
       strFile = TemplatePath() & USE_SQL_TEMPLATE
       Set objFileSystem = CreateObject("Scripting.FileSystemObject")
       If Not(objFileSystem Is Nothing) Then
          If objFileSystem.FileExists(strFile) Then
             With objFileSystem.OpenTextFile(strFile, FOR_READING)
                  mstrContents = .ReadAll
                  .Close
             End With
          Else
             mstrContents = "-- " & Replace(cERR_TEMPLATE_NOT_FOUND, "%File%", strFile)
          End If
       Else
          mstrContents = "-- " & cFILESYSTEM_NOT_AVAILABLE
       End If
    End If
    mblnWasRead = True
 End Sub

 Private Function TemplatePath()
     Dim numPos, strResult
     strResult = modulePath()
     strResult = Left(strResult, Len(strResult) - 1) ' Remove last backslash
     numPos = InStrRev(strResult, "\")
     If numPos>0 Then
        strResult = Left(strResult, numPos - 1) ' Removes the specific script type directory
        numPos = InStrRev(strResult, "\")
        If numPos>0 Then
           strResult = Left(strResult, numPos - 1) ' Removes the Script directory
        End If
     End If
     TemplatePath = strResult & "\Template\"
 End Function
End Class

Function AddSlashesTo(ByVal strInput)
    Dim strOutput, blnBinary, numCount, strChr
    strOutput = ""
    blnBinary = False
    For numCount = 1 To Len(strInput)
        strChr = Mid(strInput, numCount, 1)
        Select Case strChr
               Case chr(13): strChr = "\n"
               Case chr(10): strChr = "\r"
               Case chr(9): strChr = "\t"
               Case chr(8): strChr = "\b"
               Case "'": strChr = "''" ' ANSI Syntax
               Case "\": strChr = "\\"
               Case chr(0): strChr = "\0"
               Case chr(26): strChr = "\Z"
               Case Else:
                    If Asc(strChr)<32 Or Asc(strChr)>126 Then
                       blnBinary = True
                    End If
        End Select
        strOutput = strOutput & strChr
    Next
    If blnBinary Then
       strOutput = ToHex(strInput)
    Else
       strOutput = "'" & strOutput & "'"
    End If
    AddSlashesTo = strOutput
End Function

Sub AddSlashesToSelection()
    Dim strInput
    With newEditor()
         .assignActiveEditor()
         strInput = AddSlashesTo(.selText())
         .selText(strInput)
    End With
End Sub

Sub BeautifySql()
    With newEditor
         .assignActiveEditor
         If .selText = "" Then
            .text(WithKeywordsInUpperCase(.text, SqlCommands))
         Else
            .selText(WithKeywordsInUpperCase(.selText, SqlCommands))
         End If
    End With
End Sub

Sub LogToSql()
    Const CLEANUP_EXPRESSION = "^([0-9]{6,8}\s[0-2]{0,1}[0-9]\:[0-5][0-9]\:[0-5][0-9]){0,1}[\s]{1,}[0-9]{1,}\s(Init DB[^$]{1,}|Connect[^$]{1,}|Query[\s]{0,}|Quit)"
    ' ToDo: For each line, replace this RegExp with "" (Dikkie Dik, Tue, 8 August 2006 18:18:34 GMT)
End Sub

Function ConnectForDatabase(ByVal strConnect, ByVal strDatabase)
    Dim numStartPos, numEndPos
    If strDatabase<>"" Then
       numEndPos = 0
       numStartPos = InStr(1,strConnect,"DATABASE=")
       If numStartPos>0 Then
          numEndPos = InStr(numStartPos+1,strConnect,";")
       End If
       If numEndPos>0 Then
          strConnect = Left(strConnect, numStartPos - 1) & "DATABASE=" & strDatabase & Mid(strConnect, numEndPos)
       End If
    End If
    ConnectForDatabase = strConnect
End Function

Dim gedtLog

Sub DebugPrint(ByVal strMessage)
    If IsEmpty(gedtLog) Then
       Set gedtLog = newEditor()
       gedtLog.assignLog()
    End If
    gedtLog.appendText strMessage & LINE_END
End Sub

Sub DoSql(ByVal strSql, ByVal strConnect, ByVal objOutput)
    Set mobjConnection = new clsAdoConnection
    If mobjConnection.TryToConnect(strConnect) Then
       SplitAndExecute mobjConnection, strSql, objOutput
    Else
       Set mobjConnection = new clsDaoConnection
       If mobjConnection.TryToConnect(strConnect) Then
          SplitAndExecute mobjConnection, strSql, objOutput
       Else
          MsgBox cERR_CANNOT_CONNECT & strConnect
       End If
    End If
End Sub

Sub EditMe()
    newEditor.openFile moduleFileName(module_name)
End Sub

Sub ExecuteHighlightedSqlToCsv()
    Dim strText, strConnect
    strText = SelectedText()
    strConnect = OdbcConnect()
    DoSql strText, strConnect, New clsCsvRecordWriter
End Sub

Sub ExecuteHighlightedSqlToHtml()
    Dim strText, strConnect
    strText = SelectedText()
    strConnect = OdbcConnect()
    DoSql strText, strConnect, New clsTableRecordWriter
End Sub

Sub ExecuteHighlightedSqlToInserts()
    Dim strText, strConnect
    strText = SelectedText()
    strConnect = OdbcConnect()
    DoSql strText, strConnect, New clsSqlInsertRecordWriter
End Sub

Sub ExecuteHighlightedSqlToLastField()
    Dim strText, strConnect
    strText = SelectedText()
    strConnect = OdbcConnect()
    DoSql strText, strConnect, New clsLastFieldRecordWriter
End Sub

Sub ExecuteHighlightedSqlToTabs()
    Dim strText, strConnect
    strText = SelectedText()
    strConnect = OdbcConnect()
    DoSql strText, strConnect, New clsTabbedRecordWriter
End Sub

Sub ExecuteHighlightedSqlToUpdates()
    Dim strText, strConnect
    strText = SelectedText()
    strConnect = OdbcConnect()
    DoSql strText, strConnect, New clsSqlUpdateRecordWriter
End Sub

Sub ExtractTables()
    With New clsResultGatherer
         .Process "\s(FROM|JOIN)\s+(\w+)", New clsJoinMatchProcessor
         .Process "HANDLER\s+(\w+)\s+OPEN", New clsHandlerMatchProcessor
         .Process "\b(ALTER|DROP|CREATE)\b.TABLE\b.(IF\b.EXISTS\b.){0,1}([a-z0-9_\-]){1,}", New clsLastWordMatchProcessor
         .Process "\b(INSERT|REPLACE)\b.INTO\b.([a-z0-9_\-])*", New clsLastWordMatchProcessor
         .Process "\bUPDATE\b.([a-z0-9_\-])*\b.SET\b", New clsUpdateMatchProcessor
         .Process "\bDELETE\b.FROM\b.([a-z0-9_\-]*)\b", New clsDeleteMatchProcessor
         .Process "\bTRUNCATE\b.([a-z0-9_\-]*)", New clsLastWordMatchProcessor
         .WriteTo New clsArrayWriter
    End With
End Sub

Sub Init
    With New clsAccesskeyGenerator
         addMenuItem .WithAccessKey(cMENU_ADD_SLASHES), "&" & module_name, "AddSlashesToSelection"
         If ODBC_CONNECTION<>"" Then
            addMenuItem .WithAccessKey(cMENU_SQL_TO_CSV), "&" & module_name, "ExecuteHighlightedSqlToCsv"
            addMenuItem .WithAccessKey(cMENU_SQL_TO_HTML), "&" & module_name, "ExecuteHighlightedSqlToHtml"
            addMenuItem .WithAccessKey(cMENU_SQL_TO_TABS), "&" & module_name, "ExecuteHighlightedSqlToTabs"
            addMenuItem .WithAccessKey(cMENU_SQL_TO_INSERTS), "&" & module_name, "ExecuteHighlightedSqlToInserts"
            addMenuItem .WithAccessKey(cMENU_SQL_TO_UPDATES), "&" & module_name, "ExecuteHighlightedSqlToUpdates"
            addMenuItem .WithAccessKey(cMENU_SQL_TO_LAST_FIELD), "&" & module_name, "ExecuteHighlightedSqlToLastField"
         End If
         addMenuItem .WithAccessKey(cMENU_EXTRACT_NAMES), "&" & module_name, "ExtractTables"
         addMenuItem .WithAccessKey(cMENU_BEAUTIFY_SQL), "&" & module_name, "BeautifySql"
         If SERVER_LOG<>"" Then
            addMenuItem .WithAccessKey(cMENU_OPEN_LOG), "&" & module_name, "OpenSqlLog"
         End If
         If ADD_EDIT_ME_TO_MENU Then
            addMenuItem "-", "&" & module_name, ""
            addMenuItem .WithAccessKey(cMENU_EDIT_ME), "&" & module_name, "EditMe"
         End If
    End With
End Sub

Function IsAlphaNumeric(ByVal strCharacter)
    strCharacter = UCase(strCharacter)
    IsAlphaNumeric = ((strCharacter>="A") And (strCharacter<="Z")) Or ((strCharacter>="0") And (strCharacter<="9"))
End Function

Function OdbcConnect()
    Dim numPos
    If mstrCurrentDatabase="" Then
       numPos = InStr(1, ODBC_CONNECTION, "DATABASE=")'Read default from ODBC connect string
       If numPos>0 Then
          mstrCurrentDatabase = Mid(ODBC_CONNECTION, numPos + 9)
          numPos = InStr(1, mstrCurrentDatabase, ";")
          If numPos>0 Then
             mstrCurrentDatabase = Left(mstrCurrentDatabase, numPos - 1)
          End If
       End If
    End If
    mstrCurrentDatabase= InputBox(cCAPTION_DATABASE_SELECTOR & ":", cTITLE_DATABASE_SELECTOR, mstrCurrentDatabase)
    If (InStr(1, mstrCurrentDatabase, "=")<1) And (InStr(1, mstrCurrentDatabase, ";")<1) Then
       OdbcConnect = ConnectForDatabase(ODBC_CONNECTION, mstrCurrentDatabase)
    Else
       OdbcConnect = mstrCurrentDatabase
    End If
End Function

Sub OpenSqlLog()
    With newEditor()
         .openFile(SERVER_LOG)
         .setCaretPos 0, .linesCount()-1
    End With
End Sub

Function SelectedText()
    Dim strText
    With newEditor
         .assignActiveEditor
         strText = .selText
         If strText = "" Then
            strText = .text
         End If
    End With
    SelectedText = strText
End Function

Sub SplitAndExecute(ByVal objConnection, ByVal strSql, ByVal objOutput)
    With New clsSqlSplitter
         .AddSql strSql
         .Execute objConnection, objOutput
    End With
End Sub

Function SqlCommands()
    If IsEmpty(marrSqlCommands) Then
       marrSqlCommands = Split("ADD,ALTER DATABASE,ALTER TABLE,AS,ASC,AUTO_INCREMENT,COMMENT,CREATE DATABASE,CREATE INDEX,CREATE TABLE,CHECK,CONSTRAINT,DATABASE,DATE,DATETIME,DEFAULT,DELETE FROM,DESC,DESCRIBE,DROP DATABASE,DROP TABLE,FOREIGN KEY,FROM,GROUP BY,HANDLER,IF EXISTS,IF NOT EXISTS,INSERT,IN,INT,INTO,IS NOT NULL,IS NULL,KEY,LIKE,LIMIT,NULL,NOT NULL,ON DELETE CASCADE,ON DELETE NO ACTION,ON DELETE SET NULL,ON DELETE RESTRICT,ON UPDATE CASCADE,ON UPDATE NO ACTION,ON UPDATE SET NULL,ON UPDATE RESTRICT,OPEN AS,ORDER BY,PRIMARY,REFERENCES,SELECT,SET,SHOW CREATE DATABASE,SHOW CREATE TABLE,SHOW DATABASES,SHOW FULL FIELDS,SHOW INDEX,SHOW KEYS,SHOW TABLES,SHOW VARIABLES,TEXT,TINYTEXT,UNIQUE,UPDATE,USE,VALUES,VARCHAR,VERSION,WHERE", ",")
    End If
    SqlCommands = marrSqlCommands
End Function

Function ToHex(ByVal strInput)
    Dim strResult, numCount, strHex
    strResult = "0x" ' Giant hexadecimal string
    For numCount = 1 To Len(strInput)
        strHex = Hex(Asc(Mid(strInput, numCount, 1)))
        If (Len(strHex) Mod 2) = 1 Then
           strHex = "0" & strHex
        End If
        strResult = strResult & strHex
    Next
    ToHex = strResult
End Function

Function WithKeywordsInUpperCase(ByVal strText, ByRef arrKeywords)
    Dim intPosition, blnIsWholeWord, strKeyword
    Const TEXT_COMPARE = 1
    For Each strKeyword In arrKeywords
        intPosition = InStr(1, strText, strKeyword, TEXT_COMPARE)
        While intPosition>0
              If intPosition=1 Then
                 blnIsWholeWord = True
              Else
                 blnIsWholeWord = Not IsAlphaNumeric(Mid(strText, intPosition - 1, 1))
              End If
              If blnIsWholeWord Then
                 blnIsWholeWord = Not IsAlphaNumeric(Mid(strText, intPosition + Len(strKeyword), 1))
              End If
              If blnIsWholeWord Then
                 strText = Left(strText, intPosition - 1) & strKeyword & Mid(strText, intPosition + Len(strKeyword))
              End If
              intPosition = InStr(intPosition + Len(strKeyword), strText, strKeyword, TEXT_COMPARE)
        Wend
    Next
    WithKeywordsInUpperCase = strText
End Function
